
from datetime import datetime
from copy import deepcopy
import threading

class logger:

    def __init__(self, app_name, function_name):
        start_at = datetime.now()
        self._start_at = start_at
        self._app_name = app_name
        self._function_name = function_name

        self._log_console(f"Starting: {self._app_name}", "INFO", self._function_name)

    def start_function(self, function_name):
        start_at = datetime.now()
        function_logger = deepcopy(self)
        function_logger._function_name = function_name
        function_logger.start_at = start_at
        self._log_console(f"Starting function: {self._function_name}", "INFO", self._function_name)
        return function_logger

    def log_console(self, message, log_level):
        self._log_console(message, log_level, self._function_name)

    def close(self):
        self.log_console(f"Ending function: {self._function_name} - time in function: {(datetime.now() - self._start_at).total_seconds()} Seconds", "INFO")

    def _log_console(self, message, log_level, function_name):
        thread_id = threading.get_ident()
        log = f"{datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')} | {thread_id} | {log_level} | {self._app_name} | {function_name} | {message}"
        print(log)




import math

def haversine(lon1, lat1, lon2, lat2, log, r = 6371000):
    """
    Calculate the great circle distance in kilometers between two points
    on the earth (specified in decimal degrees)
    """
    log = log.start_function("haversine")

    # convert decimal degrees to radians
    lon1, lat1, lon2, lat2 = map(math.radians, [lon1, lat1, lon2, lat2])

    # haversine formula
    dlon = lon2 - lon1
    dlat = lat2 - lat1
    a = math.sin(dlat/2)**2 + math.cos(lat1) * math.cos(lat2) * math.sin(dlon/2)**2
    c = 2 * math.asin(math.sqrt(a))

    log.close()

    return c * r




from azure.storage.filedatalake import DataLakeServiceClient
import pandas as pd
from io import BytesIO
import pickle

class datalake():

    def __init__(self, credentials, file_system = "datalake"):
        with open(credentials, "rb") as file:
            storage_account_name, storage_account_key = pickle.load(file)

        # Defino la conexión al datalake
        service_client = DataLakeServiceClient(
                account_url="{}://{}.dfs.core.windows.net".format("https", storage_account_name),
                credential=storage_account_key)
        self._client = service_client.get_file_system_client(file_system=file_system)

        self._import_settings = {
            "parquet" : pd.read_parquet,
            "csv" : pd.read_csv
        }


    def import_file(self, path, filename, format):
        directory_client = self._client.get_directory_client(path)
        file_client = directory_client.get_file_client(filename)
        # descargo los datos
        download = file_client.download_file()
        downloaded_bytes = download.readall()

        return self._import_settings[format](BytesIO(downloaded_bytes))


    def upload_file(self, data, path, filename, format):
        directory_client = self._client.get_directory_client(path)

        file_client = directory_client.create_file(filename)
        if format == "parquet":
            file_contents = data.to_parquet(index=False).encode()
        elif format == "csv":
            file_contents = data.to_csv(index=False).encode()
        file_client.upload_data(file_contents, overwrite=True)
